#!/usr/bin/env node

const express = require('express');
const url = require('url');
const cassandra = require('cassandra-driver');
const bunyan = require('bunyan');
const { argv } = require('yargs')
  .options({
    port: {
      demandOption: true,
      describe: 'file server port',
      type: 'number',
    },
    dbhost: {
      demandOption: true,
      default: '127.0.0.1',
      describe: 'cassandra server',
      type: 'string',
    },
  });

const PORT = argv.port;
const CASSANDRA_HOST = argv.dbhost;
const SCHEMA_NAME = 'schema2';
const FILES_TABLE = 'files';

const log = bunyan.createLogger({
  name: 'serve.js',
  streams: [{
    level: 'info',
    stream: process.stdout,
  }],
});

// TODO: Possibly use more than one contactPoints
const client = new cassandra.Client({
  contactPoints: [CASSANDRA_HOST],
});

// TODO: Use security modules
const app = express();

async function _getFile(fileName) {
  const result = await client.execute(`SELECT content from ${SCHEMA_NAME}.${FILES_TABLE} WHERE name = ?`, [fileName]);
  return result.rows[0].content;
}

app.get('/*', async (req, res) => {
  const { pathname } = url.parse(req.url);
  try {
    const file = await _getFile(pathname);
    res.attachment(pathname);
    res.write(file);
    log.info(`File Served: ${pathname}`);
  } catch (e) { // TODO: Use 404 for non-existent file and don't log it
    res.status(500).send('Error Occurred while Fetching File');
    log.error(`Error serving file: ${e}`);
  } finally {
    res.end();
  }
});

app.listen(PORT, () => {
  log.info(`Server listening on port ${PORT}`);
});
